/*
 * Copyright (c) 1999 Greg Haerr <greg@censoft.com>
 *
 * Device-independent keyboard routines
 */
#include "device.h"
#include <Protocol/SimpleTextIn.h>
#include <Protocol/SimpleTextInEx.h>

//extern EFI_SIMPLE_TEXT_INPUT_EX_PROTOCOL  *mSimpleTextInEx;
EFI_KEY_DATA                              mEfiKeyDataBuffer;

EFI_STATUS
EFIAPI
GdAddEfiKeyData (
  EFI_KEY_DATA  *KeyData
  )
{

  CopyMem (&mEfiKeyDataBuffer, KeyData, sizeof (EFI_KEY_DATA));

  return EFI_SUCCESS;
}


/**
 * Open the keyboard.
 *
 * @return Fd (>0) on success, -1 on error, -2 if no keyboard.
 */
int
GdOpenKeyboard(void)
{
#if EFI
	return 1;
#else
	int	fd;

	if ((fd = kbddev.Open(&kbddev)) == -1)
		return -1;

	/* possible -2 return means no kbd*/
	return fd;
#endif
}

#if MW_FEATURE_TWO_KEYBOARDS
/**
 * Open the second keyboard.
 *
 * @return Fd (>0) on success, -1 on error, -2 if no keyboard.
 */
int
GdOpenKeyboard2(void)
{
	return kbddev2.Open(&kbddev2);
}
#endif

/**
 * Close the keyboard.
 */
void
GdCloseKeyboard(void)
{
#if 0
	kbddev.Close();
#if MW_FEATURE_TWO_KEYBOARDS
	kbddev2.Close();
#endif
#endif
}

/**
 * Return the possible modifiers (shift, control, etc) for the keyboard.
 *
 * modifiers: Recieves bitmask of supported modifiers.
 * curmodifiers: Recieves bitmask of current modifiers.
 */
void
GdGetModifierInfo(MWKEYMOD *modifiers, MWKEYMOD *curmodifiers)
{
#if EFI
  if (modifiers != NULL) {
    *modifiers    = 0;
  }
  if (curmodifiers != NULL) {
    *curmodifiers = 0;
  }
#else
#if MW_FEATURE_TWO_KEYBOARDS
	MWKEYMOD modifiers2;
	MWKEYMOD curmodifiers2;
#endif
	kbddev.GetModifierInfo(modifiers, curmodifiers);
#if MW_FEATURE_TWO_KEYBOARDS
	kbddev2.GetModifierInfo(&modifiers2, &curmodifiers2);
	if (modifiers)
		*modifiers |= modifiers2;
	if (curmodifiers)
		*curmodifiers |= curmodifiers2;
#endif
#endif
}

/**
 * This reads one keystroke from the keyboard, and the current state of
 * the mode keys (ALT, SHIFT, CTRL).  Returns -1 on error, 0 if no data
 * is ready, 1 if keypress, 2 if keyrelease.
 * This is a non-blocking call.  Returns -2 if ESC pressed.
 *
 * @param buf Recieves the key.
 * @param modifiers Recieves the state of the modifiers (shift etc).
 * @param scancode Recieves the scancode of the key.
 * @return -1 on error, 0 if no data is ready,
 * 1 if keypress, 2 if keyrelease.
 */
int
GdReadKeyboard(EFI_KEY_DATA *KeyData, MWKEYMOD *modifiers, MWSCANCODE *scancode)
{
#if EFI
  EFI_STATUS                                Status;
  EFI_SIMPLE_TEXT_INPUT_EX_PROTOCOL         *SimpleTextInEx;

  SimpleTextInEx = NULL;
  Status = EFI_NOT_READY;
  if (!PcdGetBool (PcdH2OFormBrowserSupported)) {
    gBS->HandleProtocol (
           gST->ConsoleInHandle,
           &gEfiSimpleTextInputExProtocolGuid,
           (VOID **) &SimpleTextInEx
           );
  }
  if (SimpleTextInEx != NULL) {
    Status = SimpleTextInEx->ReadKeyStrokeEx (SimpleTextInEx, KeyData);
  } else if (mEfiKeyDataBuffer.Key.UnicodeChar != 0 || mEfiKeyDataBuffer.Key.ScanCode != 0) {
    Status = EFI_SUCCESS;
    CopyMem (KeyData, &mEfiKeyDataBuffer, sizeof (EFI_KEY_DATA));
    ZeroMem (&mEfiKeyDataBuffer, sizeof (EFI_KEY_DATA));
  }
   /*else {
    ZeroMem (KeyData, sizeof (KeyData);
    Status = gST->ConIn->ReadKeyStroke (gST->ConIn, &KeyData->Key);
    KeyData->KeyState.KeyShiftState = 0;
    KeyData->KeyState.KeyToggleState = 0;
  }*/

  if (EFI_ERROR (Status)) {
    return 0;
  }

  *modifiers = 0;
  *scancode  = 0;

  return 1;
#else
	int result = kbddev.Read(buf, modifiers, scancode);
#if MW_FEATURE_TWO_KEYBOARDS
	if (result == 0)
		result = kbddev2.Read(buf, modifiers, scancode);
#endif
	return result;
#endif
}
